"""
文件工具函数
"""
import hashlib
import mimetypes
from pathlib import Path
from typing import Generator, Optional, BinaryIO, Union


def get_file_mime_type(file_path: Union[str, Path]) -> str:
    """
    获取文件的MIME类型
    
    Args:
        file_path: 文件路径
        
    Returns:
        MIME类型
    """
    file_path = Path(file_path)
    mime_type, _ = mimetypes.guess_type(str(file_path))
    return mime_type or "application/octet-stream"


def get_file_extension(file_name: str) -> str:
    """
    获取文件扩展名
    
    Args:
        file_name: 文件名
        
    Returns:
        文件扩展名（包含点号）
    """
    return Path(file_name).suffix.lower()


def humanize_file_size(size_bytes: int) -> str:
    """
    将文件大小转换为人类可读的格式
    
    Args:
        size_bytes: 文件大小（字节）
        
    Returns:
        人类可读的文件大小
    """
    for unit in ["B", "KB", "MB", "GB", "TB"]:
        if size_bytes < 1024.0:
            return f"{size_bytes:.2f} {unit}"
        size_bytes /= 1024.0
    return f"{size_bytes:.2f} PB"


def calculate_file_hash(file_path: Union[str, Path], algorithm: str = "sha256") -> str:
    """
    计算文件哈希值
    
    Args:
        file_path: 文件路径
        algorithm: 哈希算法（md5, sha1, sha256等）
        
    Returns:
        文件哈希值（十六进制）
    """
    file_path = Path(file_path)
    hash_obj = hashlib.new(algorithm)
    
    with open(file_path, "rb") as f:
        while chunk := f.read(8192):
            hash_obj.update(chunk)
    
    return hash_obj.hexdigest()


def split_file_chunks(
    file_obj: BinaryIO,
    chunk_size: int = 1024 * 1024,  # 默认1MB
    start_offset: int = 0
) -> Generator[tuple[bytes, int, bool], None, None]:
    """
    将文件分割成块
    
    Args:
        file_obj: 文件对象
        chunk_size: 块大小（字节）
        start_offset: 起始偏移量
        
    Yields:
        (块数据, 偏移量, 是否最后一块)
    """
    file_obj.seek(start_offset)
    offset = start_offset
    
    while True:
        chunk = file_obj.read(chunk_size)
        if not chunk:
            break
            
        is_last = len(chunk) < chunk_size
        yield chunk, offset, is_last
        
        offset += len(chunk)
        if is_last:
            break 